#!/usr/bin/env python3

import errno, json

class Errors(object):

	def __init__(self):
		return

	def __str__(self):
		return json.dumps(errno.errorcode,indent=4)

def main():
	errors = Errors()
	print errors
	return

if __name__ == '__main__': main()



#!/usr/bin/env python3

import os, re, sys, requests, hashlib, shutil, io, urllib3, logging, arrow

from urllib.parse import quote_plus, urlparse
from bs4 import BeautifulSoup as BS

from Baubles.Logger import Logger

logger = Logger()
logger.setLevel(logging.INFO)

others = [
	'boto3.resources.action',
	'boto3.resources.factory',
	'botocore.auth',
	'botocore.client',
	'botocore.configprovider',
	'botocore.credentials',
	'botocore.endpoint',
	'botocore.hooks',
	'botocore.httpsession',
	'botocore.loaders',
	'botocore.loaders',
	'botocore.parsers',
	'botocore.region',
	'botocore.regions',
	'botocore.retryhandler',
	'botocore.utils',
]

for other in others:
	logging.getLogger(other).setLevel(logging.WARNING)

# todo: https://stackoverflow.com/questions/22894211/how-to-resume-file-download-in-python

#_____________________________________________________________________________
class Downloader(object):
	'''
	cached download manager
	'''

	def __init__(self, cacheDir='.cache', fresh=False, resumable=False):
		#urllib3.disable_warnings()
		self.cacheDir = cacheDir
		self.fresh = fresh
		self.resumable = resumable
		logger.debug('cacheDir='+self.cacheDir)


	def download(self, url, file=None, username=None, password=None):
		'''
		read from disk before downloading
		'''
		logger.debug('url='+url)

		if not file:
			parts = urlparse(url)

			query = ''
			if len(parts.query):
				logger.debug('query=%s'%parts.query)
				md = hashlib.md5()
				md.update(quote_plus(parts.query).encode('utf8'))
				query = '.%s'%md.hexdigest()

			# fake file for dir like path
			paths = parts.path.split('/')
			if '.' not in paths[-1]:
				paths.append('.html')

			file = '%s/%s%s%s'%(
				self.cacheDir,
				parts.netloc,
				'/'.join(paths),
				query
			)

		logger.debug('file='+file)

		dirName = os.path.dirname(file)
		if dirName and not os.path.isdir(dirName):
			os.makedirs(dirName)

		data = None

		resume_byte_pos = 0
		if self.resumable and os.path.isfile(file):
			stat = os.stat(file)
			resume_byte_pos=stat.st_size #+1 ?
			logger.info('resume_byte_pos=%d'%resume_byte_pos)

		if self.fresh or self.resumable or not os.path.isfile(file):
			logger.info('to cache '+file)
			if username and password:
				auth=(username,password)
			else:
				auth=None

			resume_header = { 'Range': 'bytes=%d-' % resume_byte_pos}

			response = requests.get(url, verify=True, headers=resume_header, stream=True, auth=auth, allow_redirects=True)
			if not ('%d'%response.status_code).startswith('2'):
				file = '%s=%s'%(file,response.status_code)
				logger.warning('url=%s, code=%s'%(url, response.status_code))
			with open(file,'ab') as output:
				response.raw.decode_content = True
				#output.write(response.raw.read())
				shutil.copyfileobj(response.raw, output)

		else:
			logger.info('from cache '+file)

		with open(file, 'rb') as input:
			data = io.BytesIO(input.read())

		return data.getvalue()


#_____________________________________________________________________________
def main():
	downloader = Downloader(fresh=False, resumable=True)
	#now = arrow.now()
	if len(sys.argv) == 1:
		sys.stderr.write('usage: [<url>*]\n')
	else:
		for file in sys.argv[1:]:
			html = downloader.download(sys.argv[1])



#_____________________________________________________________________________
if __name__ == '__main__': main()




#!/usr/bin/env python3

import os,re,sys

_region = 'ap-southeast-2'
def region():
	'''
	the AWS credstash region
	'''
	return _region

_table = 'credential-store'
def table():
	'''
	the AWS DynamoDB credstash table name
	'''
	return _table

#________________________________________________________________________________________
class SquirrelBase(object):
	'''
	place holder to be substitided with the most efficient credstash implememtation available
	'''

	def __init__(self):
		pass

	def get(self,name):
		raise NotImplementedError()

	def put(self,name,value):
		raise NotImplementedError()

	def delete(self,name):
		raise NotImplementedError()

	def list(self):
		raise NotImplementedError()

Squirrel = SquirrelBase

try:

	#____________________________________________________________________________________
	import keychain, dialogs

	class KeyChainSquirrel(SquirrelBase):
		'''
		pythonista usage of the credstash API
		'''

		def __init__(self):
			super(KeyChainSquirrel,self).__init__()

		def get(self,name):
			value = keychain.get_password(
				table(),
				name
			)
			if not value:
				value = dialogs.text_dialog('%s ?'%name)
				self.put(name,value)
			return value

		def put(self,name,value):
			keychain.set_password(
				table(),
				name,
				value
			)

		def delete(self,name):
			keychain.delete_password(
				table(),
				name
			)
			return

		def list(self):
			return list()

	Squirrel = KeyChainSquirrel

except:

	try:
		#________________________________________________________________________________
		import credstash
		from credstash import ItemNotFound

		class SecretSquirrel(SquirrelBase):
			'''
			class to demonstrate the usage of the credstash API
			'''

			def __init__(self):
				super(SecretSquirrel,self).__init__()

			def get(self,name):
				try:
					value = credstash.getSecret(
						name,
						region=region(),
						table=table()
					)
					return value
				except ItemNotFound:
					sys.stderr.write('cant find credstash value for key "%s"\n'%name)
					sys.exit(1)

			def put(self,name,value):
				return credstash.putSecret(
					name,value,
					region=region(),
					table=table()
				)

			def delete(self,name):
				return credstash.deleteSecrets(
					name,
					region=region(),
					table=table()
				)

			def list(self):
				values = credstash.listSecrets(
					region=region(),
					table=table()
				)
				return map(lambda x:x['name'], values)

		Squirrel = SecretSquirrel

	except:
		#________________________________________________________________________________
		from subprocess import Popen, PIPE

		class SecretAgent(SquirrelBase):

			def __init__(self):
				super(SecretAgent,self).__init__()

			def execute(self,command):
				process = Popen('credstash --table %s --region=%s %s'%(table(), region(), command),shell=True,stdout=PIPE)
				results = process.stdout.readlines()
				del process
				results = map(lambda x: x.rstrip('\r').rstrip('\n'), results)
				return results

			def setup(self):
				return '\n'.join(self.execute('setup'))

			def get(self,name):
				value = '\n'.join(self.execute('get \'%s\''%name))
				return value

			def put(self,name,value):
				return '\n'.join(self.execute('put \'%s\' \'%s\''%(name,value)))

			def delete(self,name):
				return '\n'.join(self.execute('delete \'%s\''%name))

			def list(self):
				p = re.compile('^(.*\S)\s+--\sversion\s\d+$')
				items = []
				for item in self.execute('list'):
					m = p.match(item)
					if m:
						items.append(m.group(1))
				return items

		Squirrel = SecretAgent




#!/usr/bin/env python3

# PYTHON_ARGCOMPLETE_OK

import os, sys, re, json, xmltodict

from Argumental.Argue import Argue

if os.path.dirname(sys.argv[0]) == '.':
	sys.path.insert(0, '..')

from Spanners.Squirrel import Squirrel

args = Argue()


@args.command(single=True)
class SquirrelCommand(Squirrel):

	@args.attribute(short='v', flag=True)
	def verbose(self): return False

	def __init__(self):
		super().__init__()
		return

	@args.operation
	def get(self, name):
		'''
		get a KMS key

		'''
		return super().get(name)  #.rstrip('\r').rstrip('\n')

	@args.operation
	@args.parameter(name='replace', short='r', flag=True)
	def put(self, name, value, replace=False):
		'''
		put a KMS name,value key

		'''
		if replace: super().delete(name)
		return super().put(name, value)

	@args.operation
	def delete(self, name):
		'''
		delete a KMS name
		'''
		return super().delete(name)

	@args.operation
	def list(self):
		'''
		list the KMS names

		'''
		return list(super().list())


if __name__ == '__main__':
	result = args.execute()
	if result:
		if type(result) == list:
			print('\n'.join(result))
		else:
			print(result)




#!/usr/bin/env python3

# PYTHON_ARGCOMPLETE_OK

import os, re, sys, xmltodict, pandas
from collections import deque, OrderedDict

#sys.path.insert(0,'..')

from Argumental.Argue import Argue
from Spanners.Digger import Digger

args = Argue()

@args.command(single=True)
class Normaliser:
	'''
	Tool to convert spreadsheets into outlines

	| A   | B	 | C	 |
	|:----|:------|:------|
	| one | two   | four  |
	| one | two   | five  |
	| one | three | six   |
	| one | three | seven |

	'''

	@args.operation
	@args.parameter(name='file', help='the excel file to normalise')
	def normalise(self, file):
		'''
		normalize groupd content from left to right,
		expects repeated row column values to indicate that it is from the same group as above
		the output is created renaming .xls* to .opml
		'''

		if not re.compile('.*.xlsx*').match(file):
			sys.stderr.write(f' not an excel file ? {file}\n')
			return

		opml = re.sub('.xlsx*$', '.opml', file, flags=re.IGNORECASE)

		digger = Digger(file=file)
		digger.dig()
		digger.save(opml)


	@args.operation
	@args.parameter(name='file', help='the excel file')
	def markdown(self, file):
		'''
		excel 2 markdown format
		'''

		if not re.compile('.*.xlsx*').match(file):
			sys.stderr.write(f' not an excel file ? {file}\n')
			return

		md = re.sub('.xlsx*$', '.md', file, flags=re.IGNORECASE)

		df = pandas.read_excel(file)
		df.to_markdown(md, index=False)


if __name__ == '__main__': args.execute()



#!/usr/bin/env python3

import os, re, sys, xmltodict, pandas
from collections import deque, OrderedDict

class Digger:
	'''
	utility to recurse down into a parsed dataframe
	'''

	def __init__(self, dq=None, file=None, stack=None):
		'''
		class object used to create isolation between levels
		and copies the dataframe into a reduced set to remove global sharing issues
		'''

		if file:
			self.dq = pandas.read_excel(file)
			self.opml = OrderedDict([
				('opml', OrderedDict([
					('@version', '1.0'),
					('head', OrderedDict([
						('title', file),
						('expansionState', '0'),
					])),
					('body', OrderedDict([
					]))
				]))
			])
			self.stack = [ self.opml['opml']['body'] ]

		else:
			self.dq = dq.copy()
			self.stack = stack

	def pushd(self, value):
		self.putd(value)
		self.stack.append(
			self.stack[-1]['outline'][-1]
		)

	def putd(self, value):
		if 'outline' not in self.stack[-1].keys():
			self.stack[-1]['outline'] = []
		self.stack[-1]['outline'].append({
			'@text': value,
		})

	def popd(self):
		self.stack.pop()

	def dig(self, indent=''):
		columns = deque(self.dq.columns)
		if len(columns) == 0: return
		column = columns.popleft()

		for value in self.dq[column].unique():
			print(f'{indent}{value}')

			self.pushd(value)

			dqq = self.dq[self.dq[column] == value][columns]
			Digger(dq=dqq, stack=self.stack).dig(indent=f'\t{indent}')

			self.popd()

	def save(self, file):
		with open(file,'w') as output:
			print(f'\n{file}')
			xmltodict.unparse(self.opml, output, encoding='UTF8', pretty=True)





#!/usr/bin/env python3

import os,re,sys

from time import sleep
from functools import wraps

#_________________________________________________
def getRoot(fn):
	#print 'fn=', fn, ', name=', fn.__name__
	while hasattr(fn,'func_closure') and fn.func_closure:
		#print 'fn.func_closure=', fn.func_closure
		if len(fn.func_closure) == 0:
			break
		fn = fn.func_closure[0].cell_contents
	return fn

#_________________________________________________
class Executor(object):
	'''
	execution helpers:
	executor = Executor()
	'''

	def retry(self, *oargs, **okwargs):
		'''
		retry the enclosed function:

		@executor.retry(timeout=5, retries=3)
		def fnToRetry():
			raise Exception('fail')
		'''

		#print(oargs,okwargs)

		def _wrapit(fn):
			fn = getRoot(fn)

			@wraps(fn)
			def _wrapper(*args, **kwargs):

				timeout = okwargs.get('timeout', 5)
				retries = okwargs.get('retries', 3)
				#print(f'{timeout=} {retries=}')

				# execution
				while retries > 0:
					try:
						return fn(*args,**kwargs)
						break
					except:
						sys.stderr.write(f'{retries}: {sys.exc_info()[0]}\n')
						retries -= 1
						sleep(timeout)

			return _wrapper

		def _actualWrapper(fn): return _wrapit(fn)

		return _actualWrapper

def main():
	executor = Executor()

	@executor.retry(timeout=1) #, retries=2)
	def awooga():
		print('try')
		raise Exception('fail')

	awooga()

	return

if __name__ == '__main__': main()
